"{ Package: 'jv:dragonfly/asm' }"

"{ NameSpace: Smalltalk }"

AJInstruction subclass:#AJAlignmentInstruction
	instanceVariableNames:'alignTo'
	classVariableNames:''
	poolDictionaries:'AJConstants'
	category:'AsmJit-Instructions'
!

AJAlignmentInstruction comment:'I am a pseudo instruction used to align the following instruction to a multiple of a given byte number.

Example:
	asm := AJx64Assembler noStackFrame.
	
	"align the following instruction to a word (2bytes)"
	asm alignWord.
	asm inc: asm RAX.
	
	"align the following instruction to a double (4bytes)"
	asm alignDouble.
	asm inc: asm RAX.
	
	"align the following instruction to a QuadWord (8bytes)"
	asm alignQuad.
	asm inc: asm RAX.
	
	"align the following instruction to a multiple of an arbirary count"
	asm align: 64.
	asm inc: asm RAX.'
!

!AJAlignmentInstruction class methodsFor:'instance creation'!

align: byteSize
    ^ self new align: byteSize
!

alignDouble
    ^ self new alignDouble
!

alignQuad
    ^ self new alignQuad
!

alignWord
    ^ self new alignWord
! !

!AJAlignmentInstruction methodsFor:'accessing'!

align
    ^ alignTo
!

align: bytesSize
    "align the data to the given byte count"
    alignTo := bytesSize
!

name
    ^ String streamContents: [ :s|
        self printSelfOn: s]
!

printSelfOn: aStream  
    self align <= 1 ifTrue: [ ^ self ].
    
    aStream nextPut: $|.
    self align <= 8 
        ifTrue: [
            self align timesRepeat: [
                aStream nextPutAll: '----|']]
        ifFalse: [
            aStream 
                nextPutAll: (self align asString padded: #left to: 4 with: $ );
                nextPut: $|]
! !

!AJAlignmentInstruction methodsFor:'alignment'!

alignByte
    self align: 1
!

alignDouble
    self align: 4
!

alignQuad
    self align: 8
!

alignWord
    self align: 2
! !

!AJAlignmentInstruction methodsFor:'emitting code'!

emitCode: asm
    | padding |
    
    padding := self paddingForPosition: position.
    
    "new machine code: | padding |"
    machineCode := ByteArray new: padding .
!

paddingForPosition: aPositionNumber
    | padding |
    
    padding := aPositionNumber \\ self align.
    padding = 0 
        ifFalse: [ padding := self align - padding ].
        
    ^ padding
! !

!AJAlignmentInstruction methodsFor:'initialize-release'!

initialize 
    super initialize.
    self alignByte.
! !

!AJAlignmentInstruction methodsFor:'visitor'!

accept: anObject
    self shouldBeImplemented 
! !

