"{ Package: 'jv:dragonfly/asm' }"

"{ NameSpace: Smalltalk }"

Object subclass:#AJRoutineStackManager
	instanceVariableNames:'calls instructions assembler noStackFrame temps maxTemps
		extraStackBytes'
	classVariableNames:''
	poolDictionaries:''
	category:'AsmJit-StackManagement'
!

AJRoutineStackManager comment:''
!

!AJRoutineStackManager class methodsFor:'instance creation'!

new
    "return an initialized instance"

    ^ self basicNew initialize.
! !

!AJRoutineStackManager methodsFor:'as yet unclassified'!

analyzeInstructions: anInstructions assembler: asm
    
    instructions := anInstructions.
    assembler := asm.
    
    instructions do: #prepareCallAlignments.
        
    calls do: [:callInfo |
        callInfo asm: assembler.
        callInfo emitAlignmentIfNeeded ].

    instructions do: [:each | each processTempsWith: self].

    self emitPrologue.

    ^ instructions 
!

emitEpilogue: popExtraBytes assembler: asm

    asm leave.
    
    popExtraBytes > 0 ifTrue: [
        asm ret: popExtraBytes asUImm
        ]
    ifFalse: [
        asm ret.
        ].
!

newCdeclCall 
    self stackFrameCheck.
    ^ calls add: (AJCdeclCallInfo new)
!

newStdCall
    self stackFrameCheck.
    ^ calls add: (AJStdCallCallInfo new)
!

noStackFrame
    noStackFrame := true.
!

releaseTemps: count
    temps := temps - count
!

reserveExtraBytesOnStack: numBytes
    
    self stackFrameCheck.
    
    extraStackBytes := numBytes 
!

reserveTemp: anAJReserveTemp
    self stackFrameCheck.
    temps := temps + 1.
    maxTemps := maxTemps max: temps .
    
    anAJReserveTemp operand: (assembler stackFrameValueAtOffset: extraStackBytes + (temps * assembler wordSize )).
!

stackFrameCheck

    noStackFrame ifTrue: [
        self error: 'Operation requires stack frame management to be enabled for generated code'
        ].
! !

!AJRoutineStackManager methodsFor:'emitting'!

emitPrologue

    noStackFrame == true ifTrue: [ ^self ]. 
    instructions do: [:each |
        each setPrologue: (assembler instructionsFor: [ | numBytes |
            assembler 
                push: assembler EBP;
                mov: assembler ESP to: assembler EBP.
                
            numBytes := extraStackBytes.
            numBytes := numBytes + (maxTemps * assembler wordSize ).
             
            numBytes > 0 ifTrue: [
                (assembler 
                    sub: assembler ESP with: numBytes) annotation:
                        extraStackBytes asString , ' extra bytes + ' ,
                        maxTemps asString , ' temps'
                ]			
            ]).
        ]
! !

!AJRoutineStackManager methodsFor:'initialize-release'!

initialize
    self reset.
    noStackFrame := false.
!

reset
    instructions := nil.
    assembler := nil.
    calls := OrderedCollection new.
    temps := maxTemps := extraStackBytes := 0.
! !

